/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is JIM.
 *
 * The Initial Developer of the Original Code is
 * Pawel Chmielowski <prefiks@o2.pl>
 *
 * Portions created by the Initial Developer are Copyright (C) 2003
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 * Pawel Chmielowski <prefiks@o2.pl>
 *
 * ***** END LICENSE BLOCK ***** */

/**
 * services.X.Data
 *
 *
 */
ML.load("xml/xmldombuilder.js");
ML.load("xml/xmldomtraverse.js");
ML.load("xml/xmldomtraverseutils.js");

/**
 * XData
 *   "JEP0004 - Data Gathering and Reporting" implementation.
 *
 * Constructor:
 *   XData(tag)
 *
 * Methods:
 *   createDOM(root, dontCreateRootElement)
 *
 *   createResultPane(root)
 *   createForm(root)
 *   fillForm(root, item)
 *   getDataFromForm(root)
 *
 */

function XData(dom)
{
    this.result = {};

    if (dom)
	if (dom instanceof Node)
	    xmlDOMTraverse(dom, this.mainTree, this.result, "x");
	else
	    this.result = dom;
}

XData.prototype =
{
    result: null,

    createDOM: function(root, dontCreateRootElement)
    {
	xmlDOMBuild(root, this.mainTree, this.result,
		    "x", "jabber:x:data", dontCreateRootElement);

	return root;
    },

    createResultPane: function(root) {
	const XUL_NS="http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul";

	var i, j;
	var el;
	var cols;
	var childrens;
	var fields;
	var jidid;

	if (!this.result.reported)
	    return root;

	el = root.addElement(XUL_NS, "tree",
			     "seltype", "single",
			     "enableColumnDrag", "true");
	cols = el.addElement(XUL_NS, "treecols");
	childrens = el.addElement(XUL_NS, "treechildren",
				  "flex", "1");

	fields = this.result.reported.fields;
	for (i = 0; i < fields.length; i++) {
	    if (i != 0)
		cols.addElement(XUL_NS, "splitter",
				"class", "tree-splitter");
	    if (fields[i].type == "jid-single")
		jidid = i;

	    cols.addElement(XUL_NS, "treecol",
			    "flex", "1",
			    "label", fields[i].label || fields[i]["var"],
			    "id", "xdata-form-"+fields[i]["var"]);
	}
	for (i = 0; i < this.result.items.length; i++) {
	    el = childrens.addElement(XUL_NS, "treeitem").
		addElement(XUL_NS, "treerow");
	    fields = this.result.items[i].fields;

	    for (j = 0; j < fields.length; j++)
		el.addElement(XUL_NS, "treecell",
			      "label", fields[j].values[0],
			      "jid", jidid == j ? "true" : null);
	}
    },

    createForm: function(root)
    {
	const XUL_NS="http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul";

	var fields, i, j;
	var grid, child, row, el;
	var value;

	if (this.result.instructions)
	    root.addElement(XUL_NS, "description").
		addText(this.result.instructions);

	grid = root.addElement(XUL_NS, "grid",
			       "class", "xdata-form");
	child = grid.addElement(XUL_NS, "columns");
	child.addElement(XUL_NS, "column");
	child.addElement(XUL_NS, "column",
			 "flex", "1");
	child = grid.addElement(XUL_NS, "rows");

	if (this.result.reported)
	    fields = this.result.reported.fields;
	else if (this.result.fields)
	    fields = this.result.fields;

	for (i = 0; fields && i < fields.length; i++) {
	    label = fields[i].label || fields[i]["var"];

	    switch (fields[i].type) {
	    case "text-private":
		row = child.addElement(XUL_NS, "row");
		row.addElement(XUL_NS, "hbox",
			       "class", "labelbox").
		    addElement(XUL_NS, "description",
			       "class", fields[i].required ?
			       "required": null,
			       "value", label);

		row.addElement(XUL_NS, "textbox",
			       "type", "password",
			       "value", fields[i].values && fields[i].values[0],
			       "var", fields[i]["var"]);
		break;
	    case "text-multi":
		row = child.addElement(XUL_NS, "row");
		row.addElement(XUL_NS, "hbox",
			       "class", "labelbox").
		    addElement(XUL_NS, "description",
			       "class", fields[i].required ?
			       "required": null,
			       "value", label);

		row.addElement(XUL_NS, "textbox",
			       "multiline", "true",
			       "value", fields[i].values.join("\n"),
			       "var", fields[i]["var"]);
		break;
	    case "list-single":
		row = child.addElement(XUL_NS, "row");
		row.addElement(XUL_NS, "hbox",
			       "class", "labelbox").
		    addElement(XUL_NS, "description",
			       "class", fields[i].required ?
			       "required": null,
			       "value", label);

		el = row.addElement(XUL_NS, "menulist",
				    "var", fields[i]["var"]).
		    addElement(XUL_NS, "menupopup");

		value = fields[i].values && fields[i].values[0];

		for (j = 0; j < fields[i].options.length; j++)
		    el.addElement(XUL_NS, "menuitem",
				  "label",
				  fields[i].options[j].label ||
				  fields[i].options[j].values[0],
				  "selected",
				  fields[i].options[j].values[0] == value ? "true" : null,
				  "value", fields[i].options[j].values[0]);
				  
		break;
	    case "list-multi":
		row = child.addElement(XUL_NS, "row");
		row.addElement(XUL_NS, "hbox",
			       "class", "labelbox").
		    addElement(XUL_NS, "description",
			       "class", fields[i].required ?
			       "required": null,
			       "value", label);

		el = row.addElement(XUL_NS, "listbox",
				    "seltype", "multiple",
				    "rows", "4",
				    "var", fields[i]["var"]);
		value = { };

		for (j = 0; fields[i].values && j < fields[i].values.length; j++)
		    value[fields[i].values[j]] = 1;

		for (j = 0; j < fields[i].options.length; j++)
		    el.addElement(XUL_NS, "listitem",
				  "label",
				  fields[i].options[j].label ||
				  fields[i].options[j].values[0],
				  "rows", "3",
				  "selected",
				  value[fields[i].options[j].values[0]] ? "true" : null,
				  "value", fields[i].options[j].values[0]);
				  
		break;
	    case "boolean":
		row = child.addElement(XUL_NS, "row");
		row.addElement(XUL_NS, "hbox",
			       "class", "labelbox").
		    addElement(XUL_NS, "description",
			       "class", fields[i].required ?
			       "required": null,
			       "value", label);

		value = fields[i].values && fields[i].values[0];
		value = value == 1 || value == "true" || value == "yes";

		row.addElement(XUL_NS, "checkbox",
			       "checked", value ? "true" : "false",
			       "var", fields[i]["var"]);
				  
		break;
	    case "fixed":
		row = child.addElement(XUL_NS, "row");

		row.addElement(XUL_NS, "spacer");
		row.addElement(XUL_NS, "description").
		    addText(fields[i].values && fields[i].values[0]);
	  
		break;
	    case "hidden":
		break;

	    case "jid-single":
		break;

	    case "jid-multi":
		break;

	    default :
		row = child.addElement(XUL_NS, "row");
		row.addElement(XUL_NS, "hbox",
			       "class", "labelbox").
		    addElement(XUL_NS, "description",
			       "class", fields[i].required ?
			       "required": null,
			       "value", label);

		row.addElement(XUL_NS, "textbox",
			       "value", fields[i].values && fields[i].values[0],
			       "var", fields[i]["var"]);
		break;
	    }
	}
	return root;
    },

    fillForm: function(root, item)
    {
    },

    getDataFromForm: function(root)
    {
	var els = root.getElementsByTagName("row"), el;
	var idx = 0;
	var varhash = { };
	var val, varval;
	var i, j;
	var result = new XData();

	result.result = {
	    "type": "submit",
	    "fields": []
	};

	for (i = 0; i < this.result.fields.length; i++) {
	    varhash[this.result.fields[i]["var"]] = i;
	}

	for (i = 0; i < els.length; i++) {
	    el = els[i].childNodes[1];
	    if (el.nodeType == el.ELEMENT_NODE && (varval = el.getAttribute("var"))) {
		if ((val = varhash[varval]) > idx || val == 0) {
		    for (j = ++idx; j < val; j++)
			if (this.result.fields[j].type != "fixed")
			    result.result.fields.push({
				"var": this.result.fields[j]["var"],
				"values": this.result.fields[j].values,
			    });

		    values = [];
		    result.result.fields.push({
			"var": varval,
			"values": values,
		    });

		    switch (el.localName) {
		    case "textbox":
			values.push.apply(values, el.value.split("\n"));
			break;
		    case "menulist":
			values.push(el.value);
			break;
		    case "listbox":
			for (j = 0; j < el.selectedCount; j++)
			    values.push(el.getSelectedItem(j).value);
			break;
		    case "checkbox":
			values.push(""+el.checked);
			break;
		    default :
			result.result.pop();
		    }
		    idx = val;
		}
	    }
	}
	return result;
    },
}

XData.prototype.fieldTree = {
    __PROCESS__: new XMLDOMActionRecursive("fields"),
    __ATTRIBUTES__: {
	"label": new XMLDOMAttrActionSetProp(),
	"var": new XMLDOMAttrActionSetProp(),
	"type": new XMLDOMAttrActionSetProp(),
    },
    "desc": new XMLDOMActionSetProp(),
    "required": new XMLDOMActionSetBoolProp(),
    "value": new XMLDOMActionSetArray("values"),
    "option": {
	__PROCESS__: new XMLDOMActionRecursive("options"),
	__ATTRIBUTES__: {
	    "label": new XMLDOMAttrActionSetProp()
	},
	"value": new XMLDOMActionSetArray("values")
    }
}

XData.prototype.mainTree = {
    __ATTRIBUTES__: {
	"type": new XMLDOMAttrActionSetProp(),
    },
    "title": new XMLDOMActionSetProp(),
    "instructions": new XMLDOMActionSetProp(),
    "field": XData.prototype.fieldTree,
    "reported": {
	__PROCESS__: new XMLDOMActionRecursiveSingle("reported"),
	"field": XData.prototype.fieldTree,
    },

    "item": {
	__PROCESS__: new XMLDOMActionRecursive("items"),
	"field": XData.prototype.fieldTree,
    }
}

