ML.load("utils/exprwatcher.js");

const __JS_TMPL_NS="http://jim.mozdev.org/NS/templates";

function __BaseTemplate()
{
}

__BaseTemplate.prototype =
{
    getBuilderForTag: function(tag, stripWS)
    {
        var tmp;

	if (tag.nodeType == tag.TEXT_NODE) {
            tmp = new __TextTemplate(tag, stripWS);
            if (tmp.nodeValue)
                return tmp;

            return null
	} else if (tag.nodeType == tag.ELEMENT_NODE) 
	    if (tag.namespaceURI == __JS_TMPL_NS)
		switch (tag.localName) {
		case "value-of":
		    return new __ValueOfTemplate(tag, stripWS);
		case "choose":
		    return new __ChooseTemplate(tag, stripWS);
		case "for-each":
		    return new __ForEachTemplate(tag, stripWS);
		case "if":
		    return new __IfTemplate(tag, stripWS);
		case "text":
		    return new __TextTemplate(tag, stripWS);
		}
	    else
		return new __TagTemplate(tag, stripWS);
    },

    createWatcher: function(expression)
    {
        return new ExprWatcher(expression, this, window,
                               ["value", "property"], null, false);
    },

    createSpecWatcher: function(expression)
    {
        return new MultiExprWatcher(expression, this, window,
                                    ["value", "property"], null, false);
    },

    valueChanged: function valueChanged(value, parentProp)
    {
	this.expr.bind([value, parentProp]);
    },

    previousTemplateChanged: function previousTemplateChanged(pt)
    {
        this.previousTemplate = pt;
    },
}

function __Template(tag)
{
    var ctag, stripWS;

    this.childNodes = [];
    this.tag = tag;

    stripWS = tag.getAttributeNS(__JS_TMPL_NS, "strip-spaces") != "false";

    tag = tag.firstChild;
    while (tag) {
	if (ctag = this.getBuilderForTag(tag, stripWS)) {
	    this.childNodes.push(ctag);
	}
	tag = tag.nextSibling;
    }
}

__Template.prototype =
{
    __proto__: __BaseTemplate.prototype,

    bind: function bind(value, dom)
    {
	var i, el = this;

	for (i = 0; i < this.childNodes.length; i++)
	    el = this.childNodes[i].create(value, dom || this.tag.parentNode,
                                           el, null, null);
    },

    destroy: function(dontRemoveElements)
    {
	var i;

	for (i = 0; i < this.childNodes.length; i++)
	    this.childNodes[i].destroy(dontRemoveElements);
    },

    get previousSibling()
    {
	return this.tag.previousSibling;
    },

    get nextSibling()
    {
	return this.tag.nextSibling;
    },

    gatherResults: function()
    {
	var i;

	for (i = 0; i < this.childNodes.length; i++)
	    this.childNodes[i].gatherResults();
    }
}


function __ValueOfTemplate(tag, stripWS)
{
    this.base = __BaseTemplate;
    this.base();

    this.expr = this.createWatcher(tag.getAttribute("select"));
}

__ValueOfTemplate.prototype =
{
    __proto__: __BaseTemplate.prototype,

    create: function create(value, domRoot, previousTemplate,
                            parentObject, parentProp)
    {
	this.value = value;
	this.domRoot = domRoot;

	this.firstNode = this.lastNode = 
	    this.node = document.createTextNode("");

	domRoot.insertBefore(this.node,
			     previousTemplate ?
			     previousTemplate.nextSibling :
			     domRoot.firstChild);


	this.valueChanged(value, parentProp);

        return this;
    },

    handler: function handler()
    {
	if (this.expr.value != null)
	    this.node.replaceData(0, this.node.length, this.expr.value);
	else
	    this.node.replaceData(0, this.node.length, "");
    },

    destroy: function destroy(dontRemoveElements)
    {
	this.expr.destroy();

	if (!dontRemoveElements) {
	    this.domRoot.removeChild(this.node);
	    this.firstNode = this.lastNode = null;
	}
    },

    get previousSibling()
    {
	return this.node.previousSibling;
    },

    get nextSibling()
    {
	return this.node.nextSibling;
    },

    gatherResults: function gatherResults()
    {
    }
}

function __ChooseTemplate(tag, stripWS)
{
    var i, j, ctag, tmp, condition, work = true;

    this.base = __BaseTemplate;
    this.base();

    this.conditions = [];

    stripWS = tag.getAttributeNS(__JS_TMPL_NS, "strip-spaces") != "false" ||
        stripWS;

    for (i = 0; work && i < tag.childNodes.length; i++)
	if ((ctag = tag.childNodes[i]).nodeType == tag.ELEMENT_NODE &&
	    ctag.namespaceURI == __JS_TMPL_NS)
	    {
		if (ctag.localName == "when") {
		    if (test = ctag.getAttribute("test"))
			condition = {
			    watcher: this.createWatcher(test),
			    childNodes: []
			}
		} else if (ctag.localName == "otherwise") {
		    condition = {
			watcher: {value: true, bind: function(){}, destroy: function(){}},
			childNodes: []
		    }
		    work = false;
		}

		for (j = 0; j < ctag.childNodes.length; j++)
		    if (tmp = this.
                        getBuilderForTag(ctag.childNodes[j],
                                         ctag.getAttributeNS(__JS_TMPL_NS, "strip-spaces")
                                         != "false" || stripWS))
			condition.childNodes.push(tmp);
                this.conditions.push(condition);
	    }
    this.activeCondition = this.conditions.length;
}

__ChooseTemplate.prototype =
{
    __proto__: __BaseTemplate.prototype,

    get firstNode()
    {
	var child, i;

	if (this.activeCondition < this.conditions.length) {
	    child = this.conditions[this.activeCondition].childNodes;

	    for (i = 0; i < child.length; i++)
		if (child[i].firstNode)
		    return child[i].firstNode;
	}
	return null;
    },

    get lastNode()
    {
	var child, i;

	if (this.activeCondition < this.conditions.length) {
	    child = this.conditions[this.activeCondition].childNodes;

	    for (i = child.length-1; i >= 0; i--)
		if (child[i].lastNode)
		    return child[i].lastNode;
	}
	return null;
    },

    create: function create(value, domRoot, previousTemplate,
                            parentObject, parentProp)
    {
	var val, i;

        this.previousTemplate = previousTemplate;
        this.parentObject = parentObject;
        this.parentProp = parentProp;

	this.value = value;
	this.domRoot = domRoot;

	this.valueChanged(value, parentProp);

        return this;
    },

    valueChanged: function valueChanged(value, parentProp)
    {
	for (i = 0; i < this.conditions.length; i++)
	    this.conditions[i].watcher.bind([value, parentProp], true);

	this.handler()
    },

    handler: function handler()
    {
	var i, j;
	var condition, tmp;

	for (i = 0; i < this.conditions.length; i++)
	    if (this.conditions[i].watcher.value)
		break;

	if (i != this.activeCondition) {
            this.destroy();

	    this.activeCondition = i;

            tmp = this.previousTemplate;

	    if (i < this.conditions.length) {
		condition = this.conditions[i];
		for (j = 0; j < condition.childNodes.length; j++)
		    tmp = condition.childNodes[j].create(this.value, this.domRoot,
                                                         tmp, this.parentObject,
                                                         this.parentProp);
	    }
	}
    },

    destroy: function destroy(dontRemoveElements)
    {
	var i, condition, sibling;

	if (this.activeCondition < this.conditions.length) {
	    condition = this.conditions[this.activeCondition];
	    for (i = 0; i < condition.childNodes.length; i++)
		condition.childNodes[i].destroy(true);
	}

	if (!dontRemoveElements)
	    for (i = this.firstNode; i; i = sibling) {
		sibling = i.nextSibling;
		this.domRoot.removeChild(i);

		if (i == this.lastNode)
		    break;
	    }
    },

    get previousSibling()
    {
	return this.firstNode && this.firstNode.previousSibling;
    },

    get nextSibling()
    {
	return this.lastNode && this.lastNode.nextSibling;
    },

    gatherResults: function gatherResults()
    {
        var i, condition;

        if (this.activeCondition < this.conditions.length) {
            condition = this.conditions[this.activeCondition];

            for (i = 0; i < condition.childNodes.length; i++)
                condition.childNodes[i].gatherResults();
        }
    },

    previousTemplateChanged: function previousTemplateChanged(pt)
    {
        this.previousTemplate = pt;

	if (this.activeCondition < this.conditions.length)
	    this.conditions[this.activeCondition].childNodes[0].
                previousTemplateChanged(pt);
    },
}

function __IfTemplate(tag, stripWS)
{
    var i, tmp;

    this.base = __BaseTemplate;
    this.base();

    stripWS = tag.getAttributeNS(__JS_TMPL_NS, "strip-spaces") != "false" ||
        stripWS;

    this.expr = this.createWatcher(tag.getAttribute("test"));

    this.childNodes = [];

    for (i = 0; i < tag.childNodes.length; i++)
        if (tmp = this.getBuilderForTag(tag.childNodes[i], stripWS))
            this.childNodes.push(tmp);
}

__IfTemplate.prototype =
{
    __proto__: __BaseTemplate.prototype,

    create: function create(value, domRoot, previousTemplate,
                            parentObject, parentProp)
    {
	var val, i;

        this.previousTemplate = previousTemplate;
        this.parentObject = parentObject;
        this.parentProp = parentProp;

	this.value = value;
	this.domRoot = domRoot;

	this.valueChanged(value, parentProp);

        return this;
    },

    handler: function handler()
    {
	var i, sibling;
	var condition, tmp;

	if (!this.expr.value) {
            if (this.firstNode)
                this.destroy();
        } else {
            tmp = this.previousTemplate;

            for (i = 0; i < this.childNodes.length; i++)
                tmp = this.childNodes[i].create(this.value, this.domRoot,
                                                tmp, this.parentObject,
                                                this.parentProp);
            this.firstNode = this.childNodes[0].firstNode;
            this.lastNode = this.childNodes[this.childNodes.length-1].lastNode;
	}
    },

    destroy: function destroy(dontRemoveElements)
    {
        var i;

        for (i = 0; i < this.childNodes.length; i++)
            this.childNodes[i].destroy(true);

        for (i = this.firstNode; i; i = sibling) {
            sibling = i.nextSibling;
            this.domRoot.removeChild(i);

            if (i == this.lastNode)
                break;
        }
        this.firstNode = this.lastNode = null;
    },

    get previousSibling()
    {
	return this.firstNode && this.firstNode.previousSibling;
    },

    get nextSibling()
    {
	return this.lastNode && this.lastNode.nextSibling;
    },

    gatherResults: function gatherResults()
    {
        var i;

        for (i = 0; i < this.childNodes.length; i++)
            this.childNodes[i].gatherResults();
    },

    previousTemplateChanged: function previousTemplateChanged(pt)
    {
        this.previousTemplate = pt;

        this.childNodes[0].
            previousTemplateChanged(pt);
    },
}

function __ForEachTemplate(tag, stripWS)
{
    this.base = __BaseTemplate;
    this.base();

    this.nodesHash = {};
    this.tag = tag;

    this. stripWS = tag.getAttributeNS(__JS_TMPL_NS, "strip-spaces") != "false" ||
        stripWS;

    this.expr = this.createWatcher(tag.getAttribute("select"));
}

__ForEachTemplate.prototype =
{
    __proto__: __BaseTemplate.prototype,

    firstNodeGroup: null,
    lastNodeGroup: null,

    get firstNode()
    {
        return this.firstNodeGroup && this.firstNodeGroup[0];
    },

    get lastNode()
    {
        return this.lastNodeGroup &&
        this.lastNodeGroup[this.lastNodeGroup.length-1];
    },

    create: function create(value, domRoot, previousTemplate,
                            parentObject, parentProp)
    {
	var thisObj = this;

	this.value = value;
	this.domRoot = domRoot;

        this.previousTemplate = previousTemplate;
        this.parentObject = parentObject;
        this.parentProp = parentProp;

	this.valueChanged(value, parentProp);

        if (!this.interval)
            this.interval = setInterval(function(){thisObj.handler()}, 1000);

        return this;
    },

    handler: function handler(obj, prop, oldVal, curVal)
    {
	var i, j, val;
	var nodes, pt;

        try {
            val = this.expr.value;

            if (obj && obj == val) {
                nodes = this.nodesHash[prop];

                if (curVal == null) {
                    if (nodes.prev)
                        nodes.prev.next = nodes.next;
                    else
                        this.firstNodeGroup = nodes.next;

                    if (nodes.next) {
                        nodes.next.prev = nodes.prev;
                        if (nodes.prev)
                            nodes.next.
                                previousTemplateChanged(nodes.prev[nodes.prev.
                                                                   length - 1]);
                        else
                            nodes.next.previousTemplateChanged(null);
                    } else
                        this.lastNodeGroup = nodes.prev;

                    __unregisterHandler(val, prop, this);

                    for (i = 0; i < nodes.length; i++)
                        nodes[i].destroy();

                    delete this.nodesHash[prop];
                } else
                    for (i = 0; i < nodes.length; i++)
                        nodes[i].valueChanged(curVal, prop);
                return;
            }

            for (i in val)
                if (val[i] != null && this.nodesHash[i] == null &&
                    typeof(val[i]) != "function") {

                    nodes = [];

                    __registerHandler(val, i, this);

                    for (j = 0; j < this.tag.childNodes.length; j++)
                        if (pt = this.getBuilderForTag(this.tag.childNodes[j],
                                                       this.stripWS))
                            nodes.push(pt);

                    pt = this.lastNode || this.previousTemplate;

                    for (j = 0; j < nodes.length; j++)
                        pt = nodes[j].create(val[i], this.domRoot,
                                             pt, val, i);

                    nodes.prev = this.lastNodeGroup;
                    if (!this.firstNodeGroup)
                        this.firstNodeGroup = nodes;
                    this.lastNodeGroup = nodes;

                    this.nodesHash[i] = nodes;
                }
        } catch (ex) {
	    clearInterval(this.interval);
	    this.interval = null;
	    throw ex;
	}
    },

    destroy: function destroy(dontRemoveElements)
    {
	var i, nodes;

	if (this.interval)
	    clearInterval(this.interval);

        for (nodes = this.firstNodeGroup; nodes; nodes = node.next)
            for (i = 0; i < nodes.length; i++)
                nodes[i].destroy(dontRemoveElements);

	this.nodesHash = {};
        this.firstNodeGroup = this.lastNodeGroup = null;
    },

    get previousSibling()
    {
	return this.firstNode && this.firstNode.previousSibling;
    },

    get nextSibling()
    {
	return this.lastNode && this.lastNode.nextSibling;
    },

    gatherResults: function gatherResults()
    {
        var nodes;

        for (nodes = this.firstNodeGroup; nodes; nodes = node.next)
            for (i = 0; i < nodes.length; i++)
                nodes[i].gatherResults();
    },

    previousTemplateChanged: function previousTemplateChanged(pt)
    {
        this.previousTemplate = pt;

        if (this.firstNodeGroup)
            this.firstNodeGroup[0].
                previousTemplateChanged(pt);
    },
}

function __TextTemplate(tag, stripWS)
{
    var tmp;
    if (tag.nodeType != tag.TEXT_NODE)
        this.nodeValue = domToText(tag);
    else if (stripWS && tag.nodeValue.search(/^\s+$/) != -1)
        this.nodeValue = "";
    else
        this.nodeValue = tag.nodeValue;
}

__TextTemplate.prototype =
{
    create: function create(value, domRoot, previousTemplate,
                            parentObject, parentProp)
    {
	this.domRoot = domRoot;

	this.firstNode = this.lastNode = 
	    this.node = document.createTextNode(this.nodeValue);

        this.node.bindedTemplate = this;
        this.node.bindedValue = value;

	domRoot.insertBefore(this.node,
			     previousTemplate &&
			     previousTemplate.nextSibling);
                                                 
        return this;
    },

    valueChanged: function()
    {
    },

    destroy: function(dontRemoveElements)
    {
	if (!dontRemoveElements)
	    this.domRoot.removeChild(this.node);
    },

    get previousSibling()
    {
	return this.node.previousSibling;
    },

    get nextSibling()
    {
	return this.node.nextSibling;
    },

    gatherResults: function()
    {
    }
}

function __TagTemplate(tag, stripWS)
{
    var tmp;

    this.base = __BaseTemplate;
    this.base();

    this.nodeName = tag.nodeName;
    this.namespaceURI = tag.namespaceURI;
    this.childNodes = [];
    this.attributes = [];

    for (i = 0; i < tag.attributes.length; i++)
	if (this.hasExprRE.test(tag.attributes[i].nodeValue))
	    this.childNodes.push(new __AttributeTemplate(tag.attributes[i]));
	else
            with (tag.attributes[i])
                if (namespaceURI == __JS_TMPL_NS && localName == "strip-spaces")
                    stripWS = stripWS || nodeValue != "false";
                else
                    this.attributes.push([namespaceURI, nodeName, nodeValue]);

    tag = tag.firstChild;

    while (tag) {
	if (tmp = this.getBuilderForTag(tag, stripWS))
	    this.childNodes.push(tmp);
	tag = tag.nextSibling;
    }
}

__TagTemplate.prototype =
{
    __proto__: __BaseTemplate.prototype,
    hasExprRE: new RegExp("^(?:.*[^$])?\\$\\{"),

    create: function create(value, domRoot, previousTemplate,
                            parentObject, parentProp)
    {
	var i, tmp;

	this.domRoot = domRoot;

	this.firstNode = this.lastNode = 
	    this.node = document.
	    createElementNS(this.namespaceURI, this.nodeName);

	for (i = 0; i < this.attributes.length; i++)
	    this.node.setAttributeNS(this.attributes[i][0],
				     this.attributes[i][1],
				     this.attributes[i][2]);

        tmp = null;
	for (i = 0; i < this.childNodes.length; i++)
	    tmp = this.childNodes[i].create(value, this.node, tmp,
                                            parentObject, parentProp);

        this.node.bindedTemplate = this;
        this.node.bindedValue = value;

	domRoot.insertBefore(this.node,
			     previousTemplate ?
			     previousTemplate.nextSibling :
			     domRoot.firstChild);

        return this;
    },

    valueChanged: function valueChanged(value, parentProp)
    {
	for (i = 0; i < this.childNodes.length; i++)
	    this.childNodes[i].valueChanged(value, parentProp)
    },

    destroy: function(dontRemoveElements)
    {
	var i;

	for (i = 0; i < this.childNodes.length; i++)
	    this.childNodes[i].destroy(true);

	if (!dontRemoveElements)
	    this.domRoot.removeChild(this.node);
    },

    get previousSibling()
    {
	return this.node.previousSibling;
    },

    get nextSibling()
    {
	return this.node.nextSibling;
    },

    gatherResults: function()
    {
	var i;

	for (i = 0; i < this.childNodes.length; i++)
	    this.childNodes[i].gatherResults();
    }
}

function __AttributeTemplate(tag)
{
    this.base = __BaseTemplate;
    this.base();

    this.namespaceURI = tag.namespaceURI;
    this.nodeName = tag.nodeName;

    this.expr = this.createSpecWatcher(tag.nodeValue);
}

__AttributeTemplate.prototype =
{
    __proto__: __BaseTemplate.prototype,

    previousSibling: null,
    nextSibling: null,
    firstNode: null,
    lastNode: null,

    create: function create(value, domRoot, previousTemplate,
                            parentObject, parentProp)
    {
	this.domRoot = domRoot;

        this.previousTemplate = previousTemplate;
        this.parentObject = parentObject;
        this.parentProp = parentProp;

	this.valueChanged(value, parentProp);

        return this;
    },

    handler: function handler()
    {
	if (this.expr.value != null)
	    this.domRoot.setAttributeNS(this.namespaceURI,
					this.nodeName,
					this.expr.value);
	else
	    this.domRoot.removeAttributeNS(this.namespaceURI,
					   this.nodeName);
    },

    destroy: function destroy(dontRemoveElements)
    {
	this.expr.destroy();

	if (!dontRemoveElements)
	    this.domRoot.removeAttributeNS(this.namespaceURI,
					   this.nodeName);
    },

    gatherResults: function gatherResults()
    {
	if (this.expr.lvalue)
	    this.expr.setValue(this.parentObject, this.parentProp,
                               this.domRoot.getAttributeNS(this.namespaceURI,
                                                           this.nodeName));
    }
}

function bindValueToTemplate(value, dom) {
    var template;

    if (!dom || dom.namespaceURI != __JS_TMPL_NS || dom.localName != "template")
	return;

    template = new __Template(dom);
    template.bind(value);

    return template;
}
