/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is JIM.
 *
 * The Initial Developer of the Original Code is
 * Pawel Chmielowski <prefiks@o2.pl>
 *
 * Portions created by the Initial Developer are Copyright (C) 2003
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 * Pawel Chmielowski <prefiks@o2.pl>
 *
 * ***** END LICENSE BLOCK ***** */

/**
 * xml.XMLDomTraverseUtils
 *
 * Objects:
 *   XMLDOMActionSetProp(name)
 *
 *     Methods:
 *       handle(dom, actionTree, value, nodeName)
 *       build(dom, actionTree, value, nodeName, namespaceURI)
 *
 *     Properties:
 *       name
 *
 *   XMLDOMActionSetBoolProp(name)
 *
 *     Methods:
 *       handle(dom, actionTree, value, nodeName)
 *       build(dom, actionTree, value, nodeName, namespaceURI)
 *
 *     Properties:
 *       name
 *
 *   XMLDOMActionRecursive(name, createWhenEmpty)
 *
 *     Methods:
 *       handle(dom, actionTree, value, nodeName)
 *       build(dom, actionTree, value, nodeName, namespaceURI)
 *
 *     Properties:
 *       name
 *       createWhenEmpty
 *
 *   XMLDOMActionSetArray(name, createWhenEmpty)
 *
 *     Methods:
 *       handle(dom, actionTree, value, nodeName)
 *       build(dom, actionTree, value, nodeName, namespaceURI)
 *
 *     Properties:
 *       name
 *       createWhenEmpty
 *
 *   XMLDOMAttrActionSetProp(name)
 *
 *     Methods:
 *       handle(dom, value, attrName, attrValue)
 *       build(dom, value, attrName)
 *
 *     Properties:
 *       name
 *
 */
ML.load("xml/xmldomtraverse.js");
ML.load("xml/xmldomutils.js");
ML.load("xml/xmldombuilder.js");

function XMLDOMActionSetProp(name) {
    this.base = XMLDOMAction;
    this.base();
    this.name = name;
}

XMLDOMActionSetProp.prototype =
{
    __proto__: XMLDOMAction.prototype,

    handle: function(dom, actionTree, value, nodeName)
    {
	value[this.name || nodeName] = domToText(dom);
    },

    build: function(dom, actionTree, value, nodeName, namespaceURI)
    {
	var name = this.name || nodeName;

	if (name in value)
	    dom.addElement(namespaceURI, nodeName).
		addText(value[name]);
    }
}

function XMLDOMActionSetBoolProp(name) {
    this.base = XMLDOMAction;
    this.base();
    this.name = name;
}

XMLDOMActionSetBoolProp.prototype =
{
    __proto__: XMLDOMAction.prototype,

    handle: function(dom, actionTree, value, nodeName)
    {
	value[this.name || nodeName] = true;
    },

    build: function(dom, actionTree, value, nodeName, namespaceURI)
    {
	if (value[this.name || nodeName])
	    dom.addElement(namespaceURI, nodeName);
    }
}

function XMLDOMActionRecursive(name, createWhenEmpty) {
    this.base = XMLDOMAction;
    this.base();
    this.name = name;
    this.createWhenEmpty = createWhenEmpty;
}

XMLDOMActionRecursive.prototype = {
    __proto__: XMLDOMAction.prototype,

    handle: function (dom, tree, value, nodeName) {
	var val = {};
	var name = this.name || nodeName;

	if (value[name])
	    value[name].push(val);
	else
	    value[name] = [val];

	xmlDOMTraverseRec(dom, tree, val);
    },

    build: function (dom, tree, value, nodeName, namespaceURI) {
	var name = this.name || nodeName;
	var node;
	
	if (name in value)
	    for (var i = 0; i < value[name].length; i++) {
		node = dom.addElement(namespaceURI, nodeName);
		xmlDOMBuildRec(node, tree, value[name][i], namespaceURI);

		if (!this.createWhenEmpty && !node.hasChildNodes() && !node.hasAttributes())
		    dom.removeChild(node);
	    }
    }
}

function XMLDOMActionRecursiveSingle(name, createWhenEmpty) {
    this.base = XMLDOMAction;
    this.base();
    this.name = name;
    this.createWhenEmpty = createWhenEmpty;
}

XMLDOMActionRecursiveSingle.prototype = {
    __proto__: XMLDOMAction.prototype,

    handle: function (dom, tree, value, nodeName) {
	var val = {};

	value[this.name || nodeName] = val;
	xmlDOMTraverseRec(dom, tree, val);
    },

    build: function (dom, tree, value, nodeName, namespaceURI) {
	var name = this.name || nodeName;
	var node;
	
	if (name in value) {
	    node = dom.addElement(namespaceURI, nodeName);
	    xmlDOMBuildRec(node, tree, value[name], namespaceURI);

	    if (!this.createWhenEmpty && !node.hasChildNodes() && !node.hasAttributes())
		dom.removeChild(node);
	}
    }
}

function XMLDOMActionSetArray(name, createWhenEmpty) {
    this.base = XMLDOMAction;
    this.base();
    this.name = name;
    this.createWhenEmpty = createWhenEmpty;
}

XMLDOMActionSetArray.prototype = {
    __proto__: XMLDOMAction.prototype,

    handle: function (dom, tree, value, nodeName) {
	var name = this.name || nodeName;

	if (value[name])
	    value[name].push(domToText(dom));
	else
	    value[name] = [domToText(dom)];
    },

    build: function (dom, tree, value, nodeName, namespaceURI) {
	var name = this.name || nodeName;
	
	if (name in value)
	    for (var i = 0; i < value[name].length; i++) {
		if (this.createWhenEmpty || value[name][i])
		    dom.addElement(namespaceURI, nodeName).
			addText(value[name][i] || "");
	    }
    }
}

function XMLDOMAttrActionSetProp(name) {
    this.base = XMLDOMAttrAction;
    this.base();
    this.name = name;
}

XMLDOMAttrActionSetProp.prototype =
{
    __proto__: XMLDOMAttrAction.prototype,

    handle: function(dom, value, attrName, attrValue)
    {
	var name = this.name || attrName;

	value[attrName] = attrValue;
    },

    build: function(dom, value, attrName)
    {
	var name = this.name || attrName;

	if (name in value)
	    dom.setAttribute(attrName, value[name]);
    }
}
